## API For Ocean Data
# Import modules
import sys
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB')

import db as db

# Import packages
import requests
from datetime import datetime, date
import xml.etree.ElementTree as ET
import sqlite3
from sqlite3 import Error


## Places
flakk_rorvik = {'lat':'63.485294', 'lon':'10.174958', 'place':'flakk_rorvik'}
hollingsholmen_aukra = {'lat':'62.776811', 'lon':'6.943317', 'place':'holl_aukra'}

# DB file
db_file = "C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB\\oceandatadb.db"


## Query ocean data from met.no
def get_ocean_data(place):
	url_base = 'https://api.met.no/weatherapi/oceanforecast/0.9/'
	url_path = '?lat='+place.get('lat')+'&lon='+place.get('lon')
	url = url_base+url_path
	try:
	    r = requests.get(url)
	    print(r.status_code)
	    return r

	except requests.ConnectionError:
		print("failed to connect")

	return None


def create_oceandatadb_connection():
	""" create a database connection to the SQLite database
    :param db_file: oceandata database file
    :return: Connection object or None
    """
	try:
		conn = sqlite3.connect(db_file)
		return conn
	except Error as e:
		print(e)

	return None


def add_oceandatadb_row(conn, data):
	sql = ''' INSERT INTO OceanData(place,datetime,seaBottomTopography,seaIcePresence,
	meanTotalWaveDirection,significantTotalWaveHeight,seaCurrentDirection,seaCurrentSpeed,
	seaTemperature) VALUES(?,?,?,?,?,?,?,?,?) '''
	cur = conn.cursor()
	cur.execute(sql, data)
	return cur.lastrowid


def update_oceandatadb_row(conn, data):
	return None


def update_oceandata_db(place):
	# Delete table data
	# db.delete_table_records(db_file, 'OceanData')

	# Initialize variables
	point = place.get('place')
	i = 0

	# get ocean data from met.no
	oceandata = get_ocean_data(place)
	root = ET.fromstring(oceandata.text)

	# Create connection to DB
	conn = create_oceandatadb_connection()

	for forecasts in root.iter('{http://api.met.no}OceanForecast'):

		# Initialize ocean data variables
		datetime_obj = date.today()
		seaBottomTopography = 0
		seaIcePresence = 0
		meanTotalWaveDirection = 0
		significantTotalWaveHeight = 0
		seaCurrentDirection = 0
		seaCurrentSpeed = 0
		seaTemperature = 0
		# print(forecasts.tag, forecasts.attrib)

		# Update ocean data variables
		seaBottomTopography = float(forecasts.find('{http://wdb.met.no/wdbxml}seaBottomTopography').text)
		seaIcePresence = float(forecasts.find('{http://wdb.met.no/wdbxml}seaIcePresence').text)
		meanTotalWaveDirection = float(forecasts.find('{http://wdb.met.no/wdbxml}meanTotalWaveDirection').text)
		significantTotalWaveHeight = float(forecasts.find('{http://wdb.met.no/wdbxml}significantTotalWaveHeight').text)
		seaCurrentDirection = float(forecasts.find('{http://wdb.met.no/wdbxml}seaCurrentDirection').text)
		seaCurrentSpeed = float(forecasts.find('{http://wdb.met.no/wdbxml}seaCurrentSpeed').text)
		seaTemperature = float(forecasts.find('{http://wdb.met.no/wdbxml}seaTemperature').text)

		# Add datetime for data point
		datetime_str = forecasts.find('{http://wdb.met.no/wdbxml}validTime/{http://www.opengis.net/gml}TimePeriod/{http://www.opengis.net/gml}begin').text
		datetime_obj = datetime.strptime(datetime_str, '%Y-%m-%dT%H:%M:%SZ')

		# Add data point to DB
		dataDB = (point, datetime_obj, seaBottomTopography, seaIcePresence, meanTotalWaveDirection,
				  significantTotalWaveHeight, seaCurrentDirection, seaCurrentSpeed, seaTemperature)

		with conn:
			add_oceandatadb_row(conn, dataDB)


		# Termination criteria [days]
		if i > 24*7:
			break
		i = i + 1
