## Get weather data
# Import modules
import sys
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB')

import db as db

# Packages
import requests
from xml.etree import ElementTree
from datetime import datetime
import pandas
import sqlite3
from sqlite3 import Error

# DB file
db_file = "C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB\\weatherdatadb.db"


## Places
flakk_rorvik = {'lat':'63.485294', 'lon':'10.174958', 'place':'flakk_rorvik'}
hollingsholmen_aukra = {'lat':'62.776811', 'lon':'6.943317', 'place':'holl_aukra'}


def get_weather_data(place):
	url_base = 'https://api.met.no/weatherapi/locationforecast/1.9/.xml'
	url_path = '?lat='+place.get('lat')+'&lon='+place.get('lon')
	url = url_base+url_path
	try: 
		r = requests.get(url)
		print(r.status_code)
		return ElementTree.fromstring(r.content)
	except requests.ConnectionError:
		print("failed to connect")

	return None

def create_weatherdatadb_connection():
	""" create a database connection to the SQLite database
    :param db_file: oceandata database file
    :return: Connection object or None
    """
	try:
		conn = sqlite3.connect(db_file)
		return conn
	except Error as e:
		print(e)

	return None

def add_weatherdatadb_row(conn, data):
	sql = ''' INSERT INTO WeatherData(place,datetime,windSpeed,windDir) VALUES(?,?,?,?) '''
	cur = conn.cursor()
	cur.execute(sql, data)
	return cur.lastrowid

def update_weatherdata_db(place):
	# Delete table data
	# db.delete_table_records(db_file, 'WeatherData')

	# Initialize variables
	point = place.get('place')
	i = 0

	# Get weather data from met.no
	weather_info = get_weather_data(place)

	# Create connection to DB
	conn = create_weatherdatadb_connection()

	# Add weather data to DB
	for time in weather_info.iter('time'):
		if time.attrib.get('from') == time.attrib.get('to'):
			date = time.attrib.get('from')
			date = datetime.strptime(date, '%Y-%m-%dT%H:%M:%SZ')
			for forecast in time:
				for data in forecast:
					if data.tag == 'windDirection':
						wind_dir = data.attrib.get('deg')
					if data.tag == 'windSpeed':
						wind_speed = data.attrib.get('mps')
			# Add data point to DB
			dataDB = (point, date, wind_speed, wind_dir)

			with conn:
				add_weatherdatadb_row(conn, dataDB)
