## Class: Battery
from Classes.Cell import Cell

# Modules
import numpy as np


# Class definition
class Battery:
	# Class variables
	v_rated    = 943.8
	SOC_lower  = 0.20
	SOC_upper  = 0.60
	SOC_init   = SOC_upper


	# Instance variables
	def __init__(self, capacity, number_of_cells, line_input):
		# Construct battery cell
		self.cell        = Cell(Battery.SOC_init)

		# Battery specs
		self.capacity     = capacity
		self.capacity_kwh = capacity*Battery.v_rated
		self.SOC          = self.cell.SOC
		self.SOH          = self.cell.SOH
		self.line_input   = line_input
		self.EoL          = False
		self.NoC          = number_of_cells
		self.Prated       = self.cell.P_nom*self.NoC

		# Power
		self.power        = 0

		# Efficiency
		self.efficiency   = 0


	def set_battery_power(self, power):
		if self.SOC >= Battery.SOC_upper and power < 0:
			self.power = 0
		elif self.SOC <= Battery.SOC_lower and power > 0:
			self.power = 0
		else:
			if np.absolute(power) > self.Prated:
				self.power = self.Prated*np.sign(power)
			else:
				self.power = power

		# Avoid division by zero in nightlay quay
		if self.capacity > 0:
			# Set efficiency
			self.set_efficiency(power)
			if power > 0:
				cell_power = (self.power*(1/self.efficiency))/self.NoC
				self.cell.update_cell(cell_power)
			else:
				cell_power = (self.power*self.efficiency)/self.NoC
				self.cell.update_cell(cell_power)
		return None


	def update_battery_SOC(self):
		self.SOC = self.cell.SOC
		return None


	def update_battery_terminal_voltage(self):
		self.Voc = self.cell.Voc*13*22
		return None


	def update_battery_SOH(self):
		self.SOH = self.cell.SOH

		if self.SOH <= 0:
			self.EoL = True

		return None


	def set_efficiency(self, power):
		c_rate = np.absolute(power)/self.capacity_kwh
		losses = 0.00165829*c_rate**2 + 0.00942211*c_rate + 0.01628141
		self.efficiency = 1 - losses
		return None


	def get_efficiency(self, power):
		c_rate = np.absolute(power)/self.capacity_kwh
		losses = 0.00165829*c_rate**2 + 0.00942211*c_rate + 0.01628141
		return (1 - losses)