## Class: Charging Station Controller

import sys
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\Optimization')
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\ELCost')
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\External')
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB')


# Import modules
import pmp as pmp
import standard as std
import elcost as EL
import pcu
import wind
import current as curr
import db

# Import packages
import numpy  as np
from datetime import datetime as dt
from datetime import timedelta as delta
from datetime import timedelta as td
import pandas as pd


# Define variables
kW            = 10**3    # [kW]
nightlay_time = 233      # [min]
TIMESTEP      = 1/(2*60) # [h]


## Cost weightning factors ##
mu_el   = 1
mu_loss = 1
mu_batt = 1
mu_load = 1
mu      = [mu_el, mu_loss, mu_batt, mu_load]

# Get weather data
weather_db = "C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB\\weatherdatadb.db"
weather_data = db.query_all_db_data(weather_db, 'WeatherData')

# Get ocean current data
ocean_db = "C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB\\oceandatadb.db"
ocean_data = db.query_all_db_data(ocean_db, 'OceanData')


class CSController:
	# Class variabel

	# Instance variable
	def __init__(self, leg):
		# Charging station point
		self.leg = leg

		# Powers
		self.battery_power  = 0
		self.grid_power     = 0

		## Charging profile
		self.charge_profile_battery = []
		self.charge_profile_grid    = []
		self.charge_pointer         = 0

		# Cost of electricity
		self.C_EL      = 0
		self.C_EL_inst = 0

		# Cost of power loss
		self.C_loss = 0

		# Cost of grid load
		self.C_load = 0

		# Total controllable energy
		self.total_energy = 0

		# Predicted external factors
		self.predicted_pcu_load     = 0
		self.predicted_wind_load    = 0
		self.predicted_current_load = 0


	def set_cs_chrg_pwr(self, cs, index):

		# Set weekend flags
		if index == 2312:
			EL.set_weekend_flag()
			pcu.set_weekend_flag()

		if cs.battery.SOC >= cs.battery.SOC_upper:
			self.battery_power = 0
			self.grid_power    = 0
		else:
			if not self.charge_profile_battery or cs.battery.capacity == 0:
				self.battery_power = 0
				self.grid_power    = 0
			else:
				self.battery_power  = self.charge_profile_battery[self.charge_pointer]
				self.grid_power     = self.charge_profile_grid[self.charge_pointer]
				self.charge_pointer = self.charge_pointer + 1

		# Update CS data
		self.total_energy = self.total_energy + np.absolute(self.battery_power)*10**(-3)*TIMESTEP

		return None


	def set_ferry_chrg_pwr(self, cs, model, profile, index):

		# Set weekend flags
		if index == 2312:
			EL.set_weekend_flag()
			pcu.set_weekend_flag()

		if model.ferry.battery.SOC >= model.ferry.battery.SOC_upper:
			self.battery_power = 0
			self.grid_power    = profile.at[index, 'hotelPower']*kW
		else:
			if cs.battery.SOC <= cs.battery.SOC_lower:
				self.battery_power = 0
				self.grid_power    = self.charge_profile_grid[self.charge_pointer]
				self.charge_pointer = self.charge_pointer + 1
			elif cs.battery.capacity == 0 and cs.line_input.upper_power_limit == 0:
				self.battery_power = 0
				self.grid_power    = 0
			else:
				self.battery_power  = self.charge_profile_battery[self.charge_pointer]
				self.grid_power     = self.charge_profile_grid[self.charge_pointer]
				self.charge_pointer = self.charge_pointer + 1

				self.total_energy = self.total_energy + (np.absolute(self.battery_power) + self.grid_power)*10**(-3)*TIMESTEP

		return None

	## TODO: Add check for available charging power
	def calculate_new_charging_profile(self, cs, model, profile, index):
		# Reset pointer
		self.charge_pointer = 1

		if profile.at[index, 'mode'] == 'Conn':
			## Calculate charging profile for ferry charging

			t_chrg 			 = CSController.get_ferry_charging_time(profile, index)
			SOC_target       = CSController.get_ferry_soc_target(self, model, profile, index, t_chrg)
			
			## PMP Optimization ##
			if profile.at[index, 'leg'] == 0:
				## Nightlay charging
				# PMP
				U_grid, U_cs = pmp.pmp(model.ferry.battery.SOC, (SOC_target+0.002), cs.battery.SOC, dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S'), t_chrg, mu, 2, index)

				# Standard
				# U_grid, U_cs = std.standard(model.ferry.battery.SOC, SOC_target, t_chrg, 2)
			else:
				## Normal ferry charging
				# PMP
				U_grid, U_cs = pmp.pmp(model.ferry.battery.SOC, (SOC_target+0.002), cs.battery.SOC, dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S'), t_chrg, mu, 1, index)
				
				# Standard
				# U_grid, U_cs = std.standard(model.ferry.battery.SOC, SOC_target, t_chrg, 1)

			# Include hotel load
			hotel_load = profile.at[index, 'hotelPower']*kW
			U_grid = [u + hotel_load for u in U_grid]

			# Calculate for ferry power losses
			ferry_eff = model.ferry.eff_board*model.ferry.eff_trafo*model.ferry.eff_conv
			U_grid = [u*(1/ferry_eff) for u in U_grid]
			U_cs   = [u*(1/ferry_eff) for u in U_cs]

			# Calculate for CS power losses
			U_grid = [u*(1/cs.eff_board) for u in U_grid]
			U_cs   = [u*(1/(cs.eff_board*cs.eff_trafo*cs.eff_conv)) for u in U_cs]

			# Set charging profiles
			self.charge_profile_grid    = U_grid
			self.charge_profile_battery = U_cs


		elif profile.at[index, 'mode'] == 'Discon':
			# Calculate charging profile for charging station charging
			if profile.at[index, 'leg'] == 0:
				self.charge_profile_grid    = []
				self.charge_profile_battery = []
			else:
				# Calculate new charging profile
				t_chrg 			 = CSController.get_cs_charging_time(profile, index)
				SOC_target       = CSController.get_cs_soc_target(cs, profile, index, t_chrg)

				if cs.battery.SOC > 0.59:
					self.charge_profile_grid    = []
					self.charge_profile_battery = []
				else:
					## PMP Optimization ##
					U_grid, U_cs = pmp.pmp(cs.battery.SOC, (SOC_target+0.002), 0, dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S'), t_chrg, mu, 0, index)
				
					## Standard charging ##
					# U_grid, U_cs = std.standard(cs.battery.SOC, SOC_target, t_chrg, 0)

					# Calculate for charging station power losses
					U_cs = [u*(1/(cs.eff_board*cs.eff_trafo*cs.eff_conv)) for u in U_cs]

					# Construct charging profiles
					self.charge_profile_grid    = U_grid
					self.charge_profile_battery = [u*(-1) for u in U_cs]

		return None


	def get_ferry_soc_target(self, model, profile, index, t_chrg):
		t_transit = CSController.get_ferry_transit_time(profile, index)
		
		## Ferry in operation
		if t_transit > 0:
			if t_transit == 11:
				consumption = 192*kW

				# Add discharging power loss in ferry battery
				consumption = consumption/(1 - 0.0227)

			elif t_transit == 10:
				consumption = 174.3*kW

				# Add discharging power loss in ferry battery
				consumption = consumption/(1 - 0.02047)

			else:
				consumption = 143.4*kW

				# Add discharging power loss in ferry battery
				consumption = consumption/(1 - 0.02047)

			# Get time index
			time = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')
			t = time.hour + time.minute/60 + time.second/(60*60)

			# Add energy planning by cost of electricity
			# el_factor = EL.get_el_cost_planning_factor(t)
			# consumption = consumption/(1 - el_factor)

			# Add effect of PCUs
			CSController.set_predicted_ferry_pcu_load(self, time, t_chrg)
			consumption = consumption/(1 - self.predicted_pcu_load)

			# Add wind energy
			CSController.set_predicted_ferry_wind_load(self, index, t_chrg)
			wind_energy = self.predicted_wind_load*(t_transit/60)
			consumption = consumption + wind_energy

			# Add ocean current energy
			CSController.set_predicted_ferry_ocean_current_load(self, index, t_chrg)
			current_energy = self.predicted_current_load*(t_transit/60)
			consumption    = consumption + current_energy
			
			# Calculate SOC target
			SOC_init         = model.ferry.battery.SOC
			battery_capacity = model.ferry.battery.capacity_kwh

			capacity_consumed = consumption/battery_capacity
			SOC_target        = SOC_init+capacity_consumed
			return SOC_target

		## Ferry in nightlay
		else:
			return 0.60



	def get_cs_soc_target(cs, profile, index, t_chrg):
		return 0.6
		# Nightlay charging
		# if t_chrg > 2*60:
		# 	return 0.6
		# else:
		# 	SOC_nom = 0.54
		# 	# Get time index
		# 	time = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')
		# 	t = time.hour + time.minute/60 + time.second/(60*60)
		# 	# Add energy planning by cost of electricty
		# 	el_factor = EL.get_el_cost_planning_factor(t)
		# 	SOC_target = SOC_nom/(1 - el_factor)
		# 	if SOC_target < cs.battery.SOC:
		# 		return cs.battery.SOC
		# 	else:
		# 		return SOC_target


	def set_predicted_ferry_pcu_load(self, time, t_chrg):
		PCUmean = 25
		load_factor = 5

		time = time + td(minutes=t_chrg) + td(minutes=1)
		time = time.hour + time.minute/60 + time.second/(60*60)
		PCUs = pcu.get_statistical_pcus(time)

		self.predicted_pcu_load = ((PCUs - PCUmean)/PCUmean) * load_factor * 0.01
		return None


	def set_predicted_ferry_wind_load(self, index, t_chrg):
		index = index + t_chrg*2
		weather_index = int(index/(2*60))
		interp_index  = (index/(2*60)) - weather_index

		# Calculate interpolated wind speed
		wind_speed0   = weather_data.at[weather_index, 'windSpeed']
		wind_speed1   = weather_data.at[weather_index + 1, 'windSpeed']
		wind_speed    = wind_speed0 + (interp_index - 0)*((wind_speed1 - wind_speed0)/(1 - 0))

		# Calculate interpolated wind direction
		wind_dir0   = weather_data.at[weather_index, 'windDir']
		wind_dir1   = weather_data.at[weather_index+1, 'windDir']
		wind_dir    = wind_dir0 + (interp_index - 0)*((wind_dir1 - wind_dir0)/(1 - 0))

		P_Fw, P_Lw, Pw  = wind.calculate_wind_power(wind_speed, wind_dir)

		# Effect with regards to direction
		if self.leg == 1:
			self.predicted_wind_load = P_Fw*(-1)
		else:
			self.predicted_wind_load = P_Fw
		return None


	def set_predicted_ferry_ocean_current_load(self, index, t_chrg):
		index = index + t_chrg*2
		current_index = int(index/(2*60))
		interp_index  = (index/(2*60)) - current_index

		# Calculate interpolated wind speed
		current_speed0   = ocean_data.at[current_index, 'seaCurrentSpeed']
		current_speed1   = ocean_data.at[current_index+1, 'seaCurrentSpeed']
		current_speed    = current_speed0 + (interp_index - 0)*((current_speed1 - current_speed0)/(1 - 0))

		# Calculate interpolated wind direction
		current_dir0   = ocean_data.at[current_index, 'seaCurrentDirection']
		current_dir1   = ocean_data.at[current_index+1, 'seaCurrentDirection']
		current_dir    = current_dir0 + (interp_index - 0)*((current_dir1 - current_dir0)/(1 - 0))

		P_Fc, P_Lc, Pc  = curr.calculate_current_power(current_speed, current_dir)

		# Effect with regards to direction
		if self.leg == 1:
			self.predicted_current_load = P_Fc*(-1)
		else:
			self.predicted_current_load = P_Fc
		return None


	def get_ferry_charging_time(profile, index):
		# Get starting time
		while profile.at[index, 'mode'] != 'Docked':
			if profile.at[index, 'mode'] == 'Nightlay':
				break
			index = index + 1
		t0 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')

		# Get charge stop time
		while profile.at[index, 'mode'] != 'Discon':
			if index == len(profile)-1:
				break
			index = index + 1

		if index == len(profile)-1:
			t1 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S') + delta(minutes=0.5)
		else:
			t1 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')

		td = t1 - t0

		return td.seconds/60 # [min]


	def get_cs_charging_time(profile, index):
		cs_leg = profile.at[index, 'leg']
		# Get starting time
		while profile.at[index, 'mode'] != 'Transit':
			index = index + 1
		t0 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')

		# Get charge stop time
		while (profile.at[index, 'mode'] != 'Conn' or profile.at[index, 'leg'] != cs_leg):
			if index == len(profile)-1:
				break
			index = index + 1

		if index == len(profile)-1:
			t1 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S') + delta(minutes=0.5)
		else:
			t1 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')

		td = t1 - t0

		return td.seconds/60 # [min]


	def get_ferry_transit_time(profile, index):
		# Get profile length
		L = len(profile)

		# Get starting time
		while profile.at[index, 'mode']  != 'Transit':
			index = index + 1
			if index == L:
				return 0

			if profile.at[index, 'mode'] == 'Nightlay':
				return 0

		t0 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')

		# Get stopping time
		while profile.at[index, 'mode'] != 'Conn':
			index = index + 1

		t1 = dt.strptime(profile.at[index, 'datetime'], '%Y-%m-%d %H:%M:%S')

		td = t1 - t0

		return td.seconds/60 # [min]


	def update_EL_cost(self, t):
		t = t.hour + t.minute/60 + t.second/(60*60)
		if self.battery_power < 0:
			electricity_cost = EL.get_price(t)*np.absolute(self.battery_power)*10**(-3)*TIMESTEP
			tariff_cost      = EL.get_tariff_cost_sim(np.absolute(self.battery_power))
			self.C_EL        = self.C_EL + electricity_cost + tariff_cost
			self.C_EL_inst   = electricity_cost + tariff_cost
		else:
			electricity_cost = EL.get_price(t)*np.absolute(self.grid_power)*10**(-3)*TIMESTEP
			tariff_cost      = EL.get_tariff_cost_sim(np.absolute(self.grid_power))
			self.C_EL        = self.C_EL + electricity_cost + tariff_cost
			self.C_EL_inst   = electricity_cost + tariff_cost
		return None


	def update_power_loss_cost(self, cs, t):
		if cs.battery.capacity == 0:
			return None
		else:
			t = t.hour + t.minute/60 + t.second/(60*60)
			P_loss           = (1 - cs.battery.get_efficiency(cs.battery_power))*cs.battery_power
			electricity_cost = EL.get_price(t)*np.absolute(P_loss)*10**(-3)*TIMESTEP
			tariff_cost      = EL.get_tariff_cost_sim(np.absolute(P_loss))

			self.C_loss = self.C_loss + electricity_cost + tariff_cost
			return None


	def update_load_cost(self, t):
		t = t.hour + t.minute/60 + t.second/(60*60)
		if self.grid_power >= 1000*10**3 and EL.get_net_exchange(t) < 0:
			load_cost = EL.get_penalty_cost(t, self.grid_power)*TIMESTEP
		else:
			load_cost = 0
		# net_cost = (-EL.get_net_exchange(t)/1000)*2*EL.get_tariff_cost_sim(np.absolute(self.grid_power))
		# if net_cost < 0:
		# 	net_cost = 0
		self.C_load = self.C_load + load_cost
		return None