## Database containing ocean data

import sys
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\ELCost')

# Import modules
import elcost as EL

import pandas as pd
import sqlite3
from sqlite3 import Error
from datetime import datetime as dt
from datetime import timedelta as delta

db_file = "C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\Profiles\\Profile C\\profileC.db"
model_db_file = 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB\\model.db'

w_to_kW  = 10**(-3)


def create_connection(db_file):
    """ create a database connection to the SQLite database
        specified by db_file
    :param db_file: database file
    :return: Connection object or None
    """
    try:
        conn = sqlite3.connect(db_file)
        return conn
    except Error as e:
        print(e)
 
    return None


def create_table(conn, create_table_sql):
    """ create a table from the create_table_sql statement
    :param conn: Connection object
    :param create_table_sql: a CREATE TABLE statement
    :return:
    """
    try:
        c = conn.cursor()
        c.execute(create_table_sql)
    except Error as e:
        print(e)


def create_db():
    db_file = "C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\DB\\model.db"
 
    sql_create_table = """ CREATE TABLE IF NOT EXISTS ExternalData (
                                        id integer PRIMARY KEY,
                                        datetime datetime NOT NULL,
                                        ferryPCU REAL,
                                        ferryWind REAL,
                                        ferryCurrent REAL,
                                        csNPCU REAL,
                                        csNWind REAL,
                                        csNCurrent REAL,
                                        csSPCU REAL,
                                        csSWind REAL,
                                        csSCurrent REAL
                                    ); """

    # create a database connection
    conn = create_connection(db_file)
    if conn is not None:
        # create projects table
        create_table(conn, sql_create_table)
    else:
        print("Error! cannot create the database connection.")


def query_all_db_data(db_file, table):
    conn = create_connection(db_file)
    with conn:
        data = pd.read_sql('SELECT * FROM '+table, conn)

    return data


def delete_sql_table(db_file, table):
    conn = create_connection(db_file)
    with conn:
        cur  = conn.cursor()
        dropTableStatement = "DROP TABLE "+table
        cur.execute(dropTableStatement)
    return None


def delete_table_records(db_file, table):
    conn = create_connection(db_file)
    with conn:
        cur  = conn.cursor()
        cur.execute('DELETE FROM '+table)
    return None


def update_dash_db(model, profile, index):
    ferry = model.ferry
    cs_n0 = model.cs[0]
    cs_n  = model.cs[1]
    cs_s  = model.cs[2]

    # Get time and el-price
    time = profile.at[index, 'datetime']
    t    = dt.strptime(time, '%Y-%m-%d %H:%M:%S')
    t    = t.hour + t.minute/60 + t.second/(60*60)
    el_price = EL.get_price(t)

    ferry_data    = (time, ferry.mode, int(profile.at[index, 'leg']), ferry.controller.gen_set_power*w_to_kW, ferry.battery.power*w_to_kW, ferry.controller.propulsion_power*w_to_kW, ferry.controller.hotel_load*w_to_kW, ferry.controller.total_load*w_to_kW, ferry.battery.SOC, ferry.battery.SOH) 
    cs_n_data     = (time, cs_n.controller.battery_power*w_to_kW, cs_n.controller.grid_power*w_to_kW, cs_n.battery.SOC, cs_n.battery.SOH)
    cs_s_data     = (time, cs_s.controller.battery_power*w_to_kW, cs_s.controller.grid_power*w_to_kW, cs_s.battery.SOC, cs_s.battery.SOH)
    el_cost_data  = (time, el_price, cs_n0.controller.C_EL_inst, cs_n0.controller.C_EL, cs_n.controller.C_EL_inst, cs_n.controller.C_EL, cs_s.controller.C_EL_inst, cs_s.controller.C_EL)
    external_data = (time, ferry.controller.pcu_load, ferry.controller.wind_load*w_to_kW, ferry.controller.current_load*w_to_kW, cs_n.controller.predicted_pcu_load, cs_n.controller.predicted_wind_load*w_to_kW, cs_n.controller.predicted_current_load*w_to_kW, cs_s.controller.predicted_pcu_load, cs_s.controller.predicted_wind_load*w_to_kW, cs_s.controller.predicted_current_load*w_to_kW)

    conn = create_connection(model_db_file)
    with conn:
         # Update ferry db
        sql = ''' INSERT INTO Ferry(datetime,mode,leg,genSetPower,batteryPower,propulsionPower,hotelLoad,totalLoad,batterySOC,batterySOH)
                    VALUES(?,?,?,?,?,?,?,?,?,?) '''
        cur = conn.cursor()
        cur.execute(sql, ferry_data)

        # Update CS-N db
        sql = ''' INSERT INTO cs_n(datetime,batteryPower,gridPower,batterySOC,batterySOH)
                    VALUES(?,?,?,?,?) '''
        cur = conn.cursor()
        cur.execute(sql, cs_n_data)

        # Update CS-S db
        sql = ''' INSERT INTO cs_s(datetime,batteryPower,gridPower,batterySOC,batterySOH)
                    VALUES(?,?,?,?,?) '''
        cur = conn.cursor()
        cur.execute(sql, cs_s_data)

        # Update ELCOST db
        sql = ''' INSERT INTO el_cost(datetime,elPrice,elCostCSN0,accElCostCSN0,elCostCSN,accElCostCSN,elCostCSS,accElCostCSS)
                    VALUES(?,?,?,?,?,?,?,?) '''
        cur = conn.cursor()
        cur.execute(sql, el_cost_data)

        # Update External db
        sql = ''' INSERT INTO ExternalData(datetime,ferryPCU,ferryWind,ferryCurrent,csNPCU,csNWind,csNCurrent,csSPCU, csSWind, csSCurrent)
                    VALUES(?,?,?,?,?,?,?,?,?,?) '''
        cur = conn.cursor()
        cur.execute(sql, external_data)

    return None

