# -*- coding: utf-8 -*-
import math
import json
import pandas as pd

import dash
from dash.dependencies import Input, Output, State
import dash_core_components as dcc
import dash_html_components as html

import plotly.plotly as py
from plotly import graph_objs as go

from app import app
from data_manager import Manager

cs_data = Manager.get_cs_s_data()

layout = [
    html.Div(
        [
            html.Div(
                [
                    html.P("Operational Profile"),

                    dcc.Graph(
                        id='op_profile',
                            figure={
                                'data': [
                                    go.Scatter(
                                        x=cs_data['datetime'],
                                        y=cs_data['batterySOC'],
                                        mode='lines',
                                        opacity=0.7,
                                        marker={
                                            'size': 15,
                                            'line': {'width': 0.5, 'color': 'white'}
                                        },
                                    )
                                ],
                                'layout': go.Layout(
                                    xaxis={'title': 'Time'},
                                    yaxis={'title': 'Battery SoC'}
                                )
                            }
                        ),
                ],
                className="twelve columns chart_div",
            ),
        ],
        className="row",
        style={"marginTop": "5px"},
    ),

    ## First subfigure
    html.Div([

        html.Div([
            dcc.Dropdown(
                id='yaxis_data',
                options=[{'label': i, 'value': i} for i in list(cs_data)],
                value='batterySOC'
            ),
        ],
        style={'width': '48%', 'display': 'inline-block'}),

        html.Div([
            dcc.Dropdown(
                id='yyaxis_data',
                options=[{'label': i, 'value': i} for i in ['None']+list(cs_data)],
                value='None'
            ),
        ],style={'width': '48%', 'float': 'right', 'display': 'inline-block'})
    ], className='dropdown',
       style={'marginTop':'200px'},
    ),

    dcc.Graph(id='first_cs_s_sub_figure'),

]

@app.callback(
    dash.dependencies.Output('first_cs_s_sub_figure', 'figure'),
    [dash.dependencies.Input('yaxis_data', 'value'),
     dash.dependencies.Input('yyaxis_data', 'value')])
def update_first_cs_s_sub_figure(yaxis_name, yyaxis_name):
    x  = cs_data['datetime']
    y1 = cs_data[yaxis_name]
    if yyaxis_name == 'None':
        trace1 = go.Scatter(
        x = x,
        y = y1,
        mode = 'lines',
        visible = False,
        name = yyaxis_name
    )

    else:
        y2 = cs_data[yyaxis_name] 
        trace1 = go.Scatter(
        x = x,
        y = y2,
        mode = 'lines',
        name = yyaxis_name
    )

    # Create traces
    trace0 = go.Scatter(
        x = x,
        y = y1,
        mode = 'lines',
        name = yaxis_name
    )
    

    return {'data': [trace0, trace1]}