# -*- coding: utf-8 -*-
import dash
from dash.dependencies import Input, Output
import dash_core_components as dcc
import dash_html_components as html

from app import app
from data_manager import Manager
from apps import ferry, cs_s, cs_n, el_cost, battery_info, external

import pandas as pd


app.layout = html.Div(
    [
        # header
        html.Div([

            html.Span("App for Smarter Charging of Battery Ferries", className='app-title'),
            
            html.Div(
                html.Img(src='https://w3.siemens.dk/home/dk/dk/cc/presse/logo/PublishingImages/siemens-logo-petrol-rgb.png',height="100%")
                ,style={"float":"right","height":"100%"})
            ],
            className="row header"
            ),

        # tabs
        html.Div([

            dcc.Tabs(
                id="tabs",
                style={"height":"20","verticalAlign":"middle"},
                children=[
                    dcc.Tab(label="Ferry", value="ferry_tab"),
                    dcc.Tab(label="North Charging Station", value="cs_n_tab"),
                    dcc.Tab(id="cs_s_tab",label="South Charging Station", value="cs_s_tab"),
                    dcc.Tab(label="Electricity cost", value="el_cost_tab"),
                    dcc.Tab(label="Battery Info", value="battery_info_tab"),
                    dcc.Tab(label="External data", value="external_data_tab"),
                ],
                value="ferry_tab",
            )

            ],
            className="row tabs_div"
            ),
       
                
        # divs that save dataframe for each tab
        html.Div(
                Manager.get_ferry_data().to_json(orient="split"),  # ferry df
                id="ferry_df",
                style={"display": "none"},
            ),
        html.Div(Manager.get_cs_n_data().to_json(orient="split"), id="cs_n_df", style={"display": "none"}), # cs_n df
        html.Div(Manager.get_cs_s_data().to_json(orient="split"), id="cs_s_df", style={"display": "none"}), # cs_s df
        html.Div(Manager.get_el_cost_data().to_json(orient="split"), id="el_cost_df", style={"display": "none"}), # cs_s df

        # Tab content
        html.Div(id="tab_content", className="row", style={"margin": "2% 3%"}),
        
        html.Link(href="https://use.fontawesome.com/releases/v5.2.0/css/all.css",rel="stylesheet"),
        html.Link(href="https://cdn.rawgit.com/plotly/dash-app-stylesheets/2d266c578d2a6e8850ebce48fdb52759b2aef506/stylesheet-oil-and-gas.css",rel="stylesheet"),
        html.Link(href="https://fonts.googleapis.com/css?family=Dosis", rel="stylesheet"),
        html.Link(href="https://fonts.googleapis.com/css?family=Open+Sans", rel="stylesheet"),
        html.Link(href="https://fonts.googleapis.com/css?family=Ubuntu", rel="stylesheet"),
        html.Link(href="https://cdn.rawgit.com/amadoukane96/8a8cfdac5d2cecad866952c52a70a50e/raw/cd5a9bf0b30856f4fc7e3812162c74bfc0ebe011/dash_crm.css", rel="stylesheet")
    ],
    className="row",
    style={"margin": "0%"},
)


@app.callback(Output("tab_content", "children"), [Input("tabs", "value")])
def render_content(tab):
    if tab == "cs_n_tab":
        return cs_n.layout
    elif tab == "ferry_tab":
        return ferry.layout
    elif tab == "cs_s_tab":
        return cs_s.layout
    elif tab == "el_cost_tab":
        return el_cost.layout
    elif tab == "battery_info_tab":
        return battery_info.layout
    elif tab == "external_data_tab":
        return external.layout
    else:
        return ferry.layout

if __name__ == "__main__":
    app.run_server(debug=True)
