## Elcost functions

# Import modules
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
import matplotlib.dates as mdates
import pandas as pd
from datetime import timedelta as td
from datetime import datetime


hours      = np.linspace(0, 23, 24)

el_weekend_flag = False

## EUR / MWh
el_working = [40.82, 40.37, 39.95, 40.19, 40.14, 41.88, 43.96, 47.74, 48.95, 46.04, 44.70, 43.68, 43.25, 42.73, 41.89, 41.70, 42.04, 42.77, 43.96, 45.08, 44.80, 43.82, 42.72, 41.15]
el_weekend = [39.57, 37.63, 37.23, 36.92, 37.16, 37.25, 38.44, 39.45, 40.28, 41.14, 40.82, 40.58, 39.78, 39.41, 39.04, 39.08, 38.98, 39.83, 41.79, 42.70, 42.94, 42.17, 41.17, 39.47]

## Convert to NOK / kWh
conversion = 9.69 * (1/1000)
el_working = [i * conversion for i in el_working]
el_weekend = [i * conversion for i in el_weekend]

# El prices during operative hours
op_working = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23]
op_weekend = [0, 1, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23]

el_working_op = []
el_weekend_op = []

for tt in op_working:
	el_working_op.append(el_working[tt])

for tt in op_weekend:
	el_weekend_op.append(el_weekend[tt])

# Average values
mean_el_working = np.mean(el_working_op)
mean_el_weekend = np.mean(el_weekend_op)
# Max values
max_el_working  = np.amax(el_working)
max_el_weekend  = np.amax(el_weekend)

# Grid net exchange
net_working = [1870.27, 2132.80,2286.47,2420.27,2337.33,1797.47,146.93,-826.53,-1092.67,-788.27,-453.20,-162.67,178.47,550.60,1125.00,1387.87,1254.93,429.73,-146.53,-336.60,-87.67,13.33,541.27,1139.93]
net_weekend = [1316.67,1843.33,2339.33,2618.00,2661.33,2714.00,2615.33,2022.33,1488.50,1283.67,1654.17,1950.33,2173.33,2382.00,2406.83,2394.00,2312.67,1992.00,1513.83,1091.17,784.50,614.67,660.50,1070.33]

def set_weekend_flag():
	global el_weekend_flag
	el_weekend_flag = True
	# print('Weekend flag (EL): ', el_weekend_flag)


def get_price(time):
	# return np.interp(time, times, prices)
	# No interpolation between hourly prices
	if el_weekend_flag == True:
		prices = el_weekend
	else:
		prices = el_working
	return prices[int(time)]



def get_prices(t0, tf):
	# Initialize paramters
	prices = []
	step   = 2

	# Get time range
	dt = tf - t0
	dt = dt.seconds/60 # [min]

	# Get prices
	for i in range(0, int(dt*step) + 1):
		t = t0 + td(minutes=(i/step))
		tt = t.hour + t.minute/60 + t.second/(60*60)
		price = get_price(tt)
		prices.append(price)

	return prices

def get_tariff_cost(u):
	fixed_comp  = 18000*(1/365)*(1/24)
	power_comp  = 612*(1/365)*(1/24)*u*(1/1000)
	energy_comp = 0.028*u*(1/1000)
	return fixed_comp+power_comp+energy_comp


def get_tariff_cost_sim(u):
	power_comp  = 612*(1/365)*(1/24)*(1/(2*60))*u*(1/1000)
	energy_comp = 0.028*u*(1/1000)*(1/(2*60))
	return power_comp + energy_comp


def get_net_exchange(time):
	if el_weekend_flag == True:
		net_exchange = net_weekend
	else:
		net_exchange = net_working
	return np.interp(time, hours, net_exchange)

def get_penalty_cost(t, u):
	penalty_power = 1000*10**3
	net_factor    = -get_net_exchange(t)/1000
	return 0 # net_factor*60*(u - penalty_power)*(1/1000)*(1/(2*60))


def get_el_cost_planning_factor(time):
	el_price = get_price(time)

	if el_weekend_flag == True:
		alpha = 0.1*mean_el_weekend*1/(max_el_weekend - mean_el_weekend)
		return -(el_price - mean_el_weekend)/mean_el_weekend * alpha
	else:
		alpha = 0.1*mean_el_working*1/(max_el_working - mean_el_working)
		return -(el_price - mean_el_working)/mean_el_working * alpha


def plot_el_cost_stat():
	# data to plot
	n_groups = 24

	# create plot
	fig, ax = plt.subplots()
	index = np.arange(n_groups)
	bar_width = 0.35
	opacity = 0.8

	rects1 = plt.bar(index, el_working, bar_width,
	alpha=opacity,
	color='b',
	label='Working days')

	rects2 = plt.bar(index + bar_width, el_weekend, bar_width,
	alpha=opacity,
	color='g',
	label='Weekend')

	plt.xlabel('Hour', fontsize='16')
	plt.ylabel('Price [NOK / kWh]', fontsize='16')
	plt.title('Electricity prices', fontsize='16')
	plt.xticks(index + bar_width, ('00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23'))
	plt.legend()

	plt.tight_layout()
	plt.grid(False)
	plt.show()


def plot_working_day_price():
	working = [40.82, 40.37, 39.95, 40.19, 40.14, 41.88, 43.96, 47.74, 48.95, 46.04, 44.70, 43.68, 43.25, 42.73, 41.89, 41.70, 42.04, 42.77, 43.96, 45.08, 44.80, 43.82, 42.72, 41.15, 41.15]
	## Convert to NOK / kWh
	conversion = 9.69 * (1/1000)
	working = [i * conversion for i in working]
	points = np.linspace(2, 5.99, 10000)
	prices = []
	for point in points:
		price = working[int(point)]
		prices.append(price)

	plt.plot(points, prices, linewidth=2.0)
	plt.grid(True)
	plt.xlabel('Hour', fontsize='16')
	plt.xticks(np.arange(2, 6.1, 1.0))
	plt.ylabel('Price [NOK/kWh]', fontsize='16')
	plt.ylim([0.38,0.42])
	plt.title('Electricity prices', fontsize='16')
	plt.show()


def plot_grid_net_exchange():
	fig, ax = plt.subplots()
	plt.plot(hours, net_working, label='Working day', linewidth='2')
	plt.plot(hours, net_weekend, label='Weekend day', linewidth='2')
	plt.xlabel('Hour', fontsize='16')
	plt.ylabel('Net exchange [MW]', fontsize='16')
	plt.title('Grid net exchange', fontsize='16')
	plt.xticks(hours, ('00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23'))
	ax.axhline(0, color='black', lw=2)
	plt.legend()
	plt.grid(True)
	plt.show()
