## Wind effect on ferry

# Modules
import numpy as np
import matplotlib.pyplot as plt


def calculate_wind_power(wind_speed, wind_dir):
	# Ferry parameters
	ferry_length    = 84
	ferry_breadth   = 16
	ferry_freeboard = 5

	# Wind conditions
	V        = wind_speed
	gamma_w  = np.radians(wind_dir)

	# Wind coefficients
	c_x = 0.50
	c_y = 0.70

	# Physical parameters
	rho = 1.225

	# Projected areas
	A_Fw = ferry_breadth*ferry_freeboard
	A_Lw = ferry_length*ferry_freeboard

	# Wind speed components
	u_w = V*np.cos(gamma_w)
	v_w = V*np.sin(gamma_w)

	# Wind coefficient approx components
	Cx = c_x*np.cos(gamma_w)
	Cy = c_y*np.sin(gamma_w)

	F_Fw = 0.5 * rho * (V**2) * Cx * A_Fw
	F_Lw = 0.5 * rho * (V**2) * Cy * A_Lw

	Fw = np.sqrt(F_Fw**2 + F_Lw**2)

	P_Fw = F_Fw*V
	P_Lw = F_Lw*V

	Pw = np.sqrt(P_Fw**2 + P_Lw**2)
	return P_Fw, P_Lw, Pw


def plot_wind_power():
	speeds = np.linspace(0.1, 15, 100)
	wind_dirs = [0, 45, 90, 150]

	for wind_dir in wind_dirs:
		Pw = []
		for speed in speeds:
			P = calculate_wind_power(speed, wind_dir)
			Pw.append(P*10**(-3))

		plt.plot(speeds, Pw, label='Direction [°]: %d'%wind_dir)

	plt.grid(True)
	plt.xlabel('Wind speed [m/s]', fontsize='16')
	plt.ylabel('Wind power [kW]', fontsize='16')
	plt.legend()
	plt.title('Wind power in sway component', fontsize='16')
	plt.show()


def plot_wind_coeffs():
	# current coefficients
	c_x = 0.50
	c_y = 0.70

	y1 = []
	y2 = []
	dirs = np.linspace(0, 360, 720)
	for gamma_w in dirs:
		gamma_w = np.radians(gamma_w)
		# current coefficient approx components
		Cx = c_x*np.cos(gamma_w)
		Cy = np.absolute(c_y*np.sin(gamma_w))
		y1.append(Cx)
		y2.append(Cy)

	plt.plot(dirs, y1, linewidth='2', label='C_w,x')
	plt.plot(dirs, y2, linewidth='2', label='C_w,y')
	plt.xlabel('Current direction [°]', fontsize='16')
	plt.ylabel('Coefficient value', fontsize='16')
	plt.title('Wind coefficient approximation values', fontsize='16')
	plt.legend()
	plt.grid(True)
	plt.show()
