# Cost function considering:
#  - Battery aging
#  - Battery power losses
#  - Cost of electricity

import sys
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\Classes')
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\ELCost')

# Modules
import elcost as el
import Cell as Cell
import numpy as np


# Parameters
c_loss = 420 # [NOK / MWh]
c_batt = 50  # Cell cost [NOK]

psi_EL   = 1 # 1
psi_loss = 1 # 0.5e3
psi_batt = 1 # 2
psi_load = 1 
psi      = [psi_EL, psi_loss, psi_batt, psi_load]

R_0    = 12*10**(-3) # [Ohm]
C_nom  = 2.5 # [Ah]
Gamma  = Cell.get_nominal_battery_life()
MW     = 10**(-6)
kW     = 10**(-3)

NoC_ferry = 205920   # Number of cells in ferry battery
NoC_cs    = 137280   # Number of cells in cs battery

# Degradation parameters
z   = 0.57 # Ah throughput dependence
eta = 152.5 # C-rate dependence
Rg  = 8.3144621 # Universal gas constant
Ea  = 31500 # Activation energy
EoL = 20 # [%] End of life condition
alpha = 2795.55
beta  = 6716.7


def get_cost(u_ferry, u_cs, I_ferry, I_cs, x_ferry, x_cs, my, t, N, mode, index):

	# Set weekend flags
	if index >= 2312:
		el.set_weekend_flag()

	Ic_ferry   = np.absolute(I_ferry) / C_nom
	Ic_cs      = np.absolute(I_cs) / C_nom
	loss_ferry = 0.00165829*Ic_ferry**2 + 0.00942211*Ic_ferry + 0.01628141
	loss_cs    = 0.00165829*Ic_cs**2 + 0.00942211*Ic_cs + 0.01628141

	load_factor = 1.2

	if mode == 0:
		## CS Charging ##
		# TERM: Cost of electricity
		J_EL_ferry = 0
		J_EL_cs    = my[0] * psi[0] * (el.get_price(t) * np.absolute(u_cs) * kW + el.get_tariff_cost(u_cs))
		J_EL       = J_EL_ferry + J_EL_cs

		# TERM: Cost of power loss
		P_loss       = loss_cs * np.absolute(u_cs)
		J_loss_ferry = 0
		J_loss_cs    = my[1] * psi[1] * (el.get_price(t) * P_loss * kW + el.get_tariff_cost(P_loss))
		J_loss       = J_loss_ferry + J_loss_cs

		# TERM: Cost of battery aging
		J_batt_ferry = 0
		J_batt_cs    = my[2] * psi[2] * (c_batt/Gamma) * Cell.get_severity_factor(x_cs, Ic_cs, 25) * np.absolute(I_cs) * NoC_cs
		J_batt       = J_batt_ferry + J_batt_cs

		# TERM: Load on grid
		if u_cs >= 1000*10**3 and el.get_net_exchange(t) < 0:
			# J_load = my[3] * psi[3] * (-el.get_net_exchange(t)/1000)*load_factor*el.get_tariff_cost(np.absolute(u_cs))
			J_load = my[3] * psi[3] * el.get_penalty_cost(t, u_cs)
		else:
			J_load = 0

	elif mode == 1:
		t_recharge = t + (N+1)/(2*60)
		if t_recharge >= 24:
			t_recharge = t_recharge - 24

		u_grid = u_ferry - u_cs

		# TERM: Cost of electricity
		J_EL_ferry = my[0] * psi[0] * (el.get_price(t) * np.absolute(u_grid) * kW + el.get_tariff_cost(u_grid))
		J_EL_cs    = my[0] * psi[0] * (el.get_price(t_recharge) * np.absolute(u_cs) * kW + el.get_tariff_cost(u_cs))
		J_EL       = J_EL_ferry + J_EL_cs
		
		# TERM: Cost of power loss
		P_loss_ferry = loss_ferry * np.absolute(u_ferry)
		P_loss_cs    = loss_cs * np.absolute(u_cs)

		J_loss_ferry = my[1] * psi[1] * (el.get_price(t) * P_loss_ferry * kW + el.get_tariff_cost(P_loss_ferry))
		J_loss_cs    = 1.12*my[1] * psi[1] * (el.get_price(t) * P_loss_cs * kW + el.get_tariff_cost(P_loss_cs))
		J_loss       = J_loss_ferry + J_loss_cs
		
		# TERM: Cost of battery aging
		J_batt_ferry = my[2] * psi[2] * (c_batt/Gamma) * Cell.get_severity_factor(x_ferry, Ic_ferry, 25) * np.absolute(I_ferry) * NoC_ferry
		J_batt_cs    = 1.12*my[2] * psi[2] * (c_batt/Gamma) * Cell.get_severity_factor(x_cs, Ic_cs, 25) * np.absolute(I_cs) * NoC_cs
		J_batt       = J_batt_ferry + J_batt_cs

		# TERM: Load on grid
		if u_grid >= 1000*10**3 and el.get_net_exchange(t) < 0:
			#J_load = my[3] * psi[3] * (-el.get_net_exchange(t)/1000)*load_factor*el.get_tariff_cost(np.absolute(u_grid))
			J_load = my[3] * psi[3] * el.get_penalty_cost(t, u_grid)
		else:
			J_load = 0

	else:
		# TERM: Cost of electricity
		J_EL_ferry = my[0] * psi[0] * (el.get_price(t) * np.absolute(u_ferry) * kW + el.get_tariff_cost(u_ferry))
		J_EL_cs    = 0
		J_EL       = J_EL_ferry + J_EL_cs
		
		# TERM: Cost of power loss
		P_loss       = loss_ferry * np.absolute(u_ferry)
		J_loss_ferry = my[1] * psi[1] * (el.get_price(t) * P_loss * kW + el.get_tariff_cost(P_loss))
		J_loss_cs    = 0
		J_loss       = J_loss_ferry + J_loss_cs
		
		# TERM: Cost of battery aging
		J_batt_ferry = my[2] * psi[2] * (c_batt/Gamma) * Cell.get_severity_factor(x_ferry, Ic_ferry, 25) * np.absolute(I_ferry) * NoC_ferry
		J_batt_cs    = 0
		J_batt       = J_batt_ferry + J_batt_cs

		# TERM: Load on grid
		if u_ferry >= 1000*10**3 and el.get_net_exchange(t) < 0:
			# J_load = my[3] * psi[3] * (-el.get_net_exchange(t)/1000)*load_factor*el.get_tariff_cost(np.absolute(u_ferry))
			J_load = my[3] * psi[3] * el.get_penalty_cost(t, u_ferry)
		else:
			J_load = 0

	# Return total cost
	# print(J_EL, J_loss, J_batt)
	return J_EL + J_loss + J_batt + J_load


def get_co_state(x_ferry, x_cs, I_ferry, I_cs, T, my):
	Ic_ferry = np.absolute(I_ferry) / C_nom
	Ic_cs    = np.absolute(I_cs) / C_nom

	# Severity factor differentiation for ferry
	c_ferry    = (-Ea + eta*Ic_ferry)/(Rg*(273.15 + T))
	diff_ferry = ( (alpha*np.exp(-c_ferry)*EoL**(-1/z) * (np.exp(-c_ferry)/(alpha*x_ferry + beta))**(-1/z - 1) ) / (z*(alpha*x_ferry + beta)**2) )

	# Severity factor differentiation for cs
	c_cs    = (-Ea + eta*Ic_cs)/(Rg*(273.15 + T))
	diff_cs = ( (alpha*np.exp(-c_cs)*EoL**(-1/z) * (np.exp(-c_cs)/(alpha*x_cs + beta))**(-1/z - 1) ) / (z*(alpha*x_cs + beta)**2) )

	return (-1) * my[2] * (c_batt / Gamma) * (np.absolute(I_ferry)*Gamma*diff_ferry*NoC_ferry + np.absolute(I_cs)*Gamma*diff_cs*NoC_cs)