# Current charging strategy

import sys
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\Classes')
sys.path.insert(0, 'C:\\Users\\noldu\\OneDrive\\NTNU\\Master thesis\\Application\\ELCost')

# Modules
import Cell as Cell
import cost as cost
import elcost as EL

## Import packages
import numpy as np
import matplotlib.pyplot as plt
from datetime import timedelta as td
from datetime import datetime

# System variables
TIMESTEP = 1/(60*2) # [h]
STEP     = 0.5      # [min]
NoC_ferry = 205920   # Number of cells in ferry battery
NoC_cs    = 137280   # Number of cells in cs battery

NoC_ferry_Ah = 720
NoC_cs_Ah    = 480

# Cost parameters
c_batt = 50  # Cell cost [NOK]
Gamma  = Cell.get_nominal_battery_life()

def standard(x0, xf, tf, mode):
	U_grid = []
	U_cs   = []
	U_grid.append(0)
	U_cs.append(0)

	# Calculate current amplitude
	cell = Cell.Cell(x0)
	N    = int(tf/STEP + 1) # Length of optimization horizon
	I_tot = (cell.capacity/(tf*(1/60))) * (x0 - xf)
	I     = [I_tot]*(N-1)

	if mode == 0:
		## CS charging ##
		for i in I:
			u = cell.Voc*i - Cell.Cell.R_0*i**2
			cell.update_cell(u)

			# Calculate for battery power losses
			loss  = 0.00165829*cell.Ic**2 + 0.00942211*cell.Ic + 0.01628141
			u_eff = 1 - loss
			u     = u/u_eff

			U_grid.append(u*(0))
			U_cs.append(-u*(1)*NoC_cs)

	elif mode == 1:
		## Ferry charging ##
		for i in I:
			u = cell.Voc*i - Cell.Cell.R_0*i**2
			cell.update_cell(u)

			# Calculate for power losses in ferry battery
			loss  = 0.00165829*cell.Ic**2 + 0.00942211*cell.Ic + 0.01628141
			u_eff = 1 - loss
			u     = u/u_eff

			if np.absolute(u*NoC_ferry) > 3042*10**3:

				u_grid = -3042*10**3/NoC_ferry
				u_cs   = u - u_grid

			else:
				u_grid = u
				u_cs   = 0

			# Calculate for power losses in CS battery
			I_cs  = cell.get_cell_current(u_cs)
			Ic_cs = cell.get_cell_c_rate(I_cs)
			loss_cs = 0.00165829*Ic_cs**2 + 0.00942211*Ic_cs + 0.01628141
			u_cs    = u_cs/(1 - loss_cs)

			U_grid.append(-u_grid*NoC_ferry)
			U_cs.append(-u_cs*NoC_ferry)

	else:
		## Nightlay charging ##
		for i in I:
			u = cell.Voc*i - Cell.Cell.R_0*i**2
			cell.update_cell(u)

			# Calculate for battery power losses
			loss  = 0.00165829*cell.Ic**2 + 0.00942211*cell.Ic + 0.01628141
			u_eff = 1 - loss
			u     = u/u_eff

			U_grid.append(-u*(1)*NoC_ferry)
			U_cs.append(u*(0))

	return U_grid, U_cs

def run_standard_plot():
	x0 = 0.454358695856
	xf = 0.5945
	t0 = datetime(2019, 4, 1, 7, 59, 30)
	tf = 26 # [min]
	## MODE
	# 0: CS charging
	# 1: Ferry charging
	# 2: Nightlay charging
	mode = 0
	x0_cs = 0.454358695856
	index = 0

	my_el   = 1
	my_loss = 1
	my_batt = 1
	my_loss = 1
	my = [my_el, my_loss, my_batt, my_loss]

	# Get optimal trajectory
	# U_opt_grid, U_opt_cs = standard(x0, xf, tf, mode)
	#print(U_opt_cs)

	pmp_pwr = [0, 446160.0, 446160.0, 446160.0, 446160.0, 446160.0, 446160.0, 446160.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0]
	pmp_pwr_ =[446160.0, 446160.0, 446160.0, 446160.0, 446160.0, 446160.0, 446160.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 411840.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 377520.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0, 343200.0]
	mean_pwr = np.mean(pmp_pwr_)
	std_pwr = [0] + [mean_pwr]*(len(pmp_pwr)-1)
	U_opt_cs = pmp_pwr
	U_opt_grid = [0]*len(pmp_pwr)


	ferry_cell = Cell.Cell(0.603190321618)
	cs_cell    = Cell.Cell(x0_cs)

	SOC_cs       = []
	SOC_ferry    = []
	U_opt_ferry  = []
	C_cs         = []
	C_ferry      = []
	total_elcost = 0
	P_loss_cost  = 0
	total_energy = 0
	total_load_cost = 0
	load_factor = 1.2

	for i in range(0, len(U_opt_grid)):
		u_cs   = U_opt_cs[i]
		u_grid = U_opt_grid[i]

		# get battery loss
		cs_cell_current = cs_cell.get_cell_current(u_cs/NoC_cs)
		cs_cell_c_rate  = cs_cell.get_cell_c_rate(cs_cell_current)
		cs_cell_power_loss = 0.00165829*cs_cell_c_rate**2 + 0.00942211*cs_cell_c_rate + 0.01628141
		u_cs_ = u_cs*(1 - cs_cell_power_loss)

		if mode == 0:
			u_ferry_ = 0
			# Update cell
			ferry_cell.update_cell(u_ferry_/NoC_ferry)
			cs_cell.update_cell(-u_cs_/NoC_cs)

			SOC_ferry.append(ferry_cell.SOC)
			SOC_cs.append(cs_cell.SOC)

			U_opt_ferry.append(u_ferry_)
			C_cs.append(cs_cell.I*NoC_cs_Ah)


		elif mode == 1:
			ferry_cell_current = ferry_cell.get_cell_current((u_cs_ + u_grid)/NoC_ferry)
			ferry_cell_c_rate  = ferry_cell.get_cell_c_rate(ferry_cell_current)
			ferry_cell_power_loss = 0.00165829*ferry_cell_c_rate**2 + 0.00942211*ferry_cell_c_rate + 0.01628141
			u_ferry_ = (u_cs_ + u_grid)*(1 - ferry_cell_power_loss)


			# Update cell
			ferry_cell.update_cell(-u_ferry_/NoC_ferry)
			cs_cell.update_cell(u_cs_/NoC_cs)
			C_cs.append(cs_cell.I*NoC_cs_Ah)

			SOC_ferry.append(ferry_cell.SOC)
			SOC_cs.append(cs_cell.SOC)

			U_opt_ferry.append(u_ferry_)
			C_ferry.append(ferry_cell.I*NoC_ferry_Ah)


		else:
			ferry_cell_current = ferry_cell.get_cell_current((u_grid)/NoC_ferry)
			ferry_cell_c_rate  = ferry_cell.get_cell_c_rate(ferry_cell_current)
			ferry_cell_power_loss = 0.00165829*ferry_cell_c_rate**2 + 0.00942211*ferry_cell_c_rate + 0.01628141
			u_ferry_ = (u_grid)*(1 - ferry_cell_power_loss)

			# Update cell
			ferry_cell.update_cell(-u_ferry_/NoC_ferry)
			cs_cell.update_cell(u_cs_/NoC_cs)

			SOC_ferry.append(ferry_cell.SOC)
			SOC_cs.append(cs_cell.SOC)

			U_opt_ferry.append(u_ferry_)
			C_ferry.append(ferry_cell.I*NoC_ferry_Ah)

		

		# Update charging performance
		t_ = t0 + td(minutes=(i/2))
		t = t_.hour + t_.minute/60 + t_.second/(60*60)

		t_recharge = t_ + td(minutes=(tf+0.5))
		t_recharge = t_recharge.hour + t_recharge.minute/60 + t_recharge.second/(60*60)

		if mode == 0:
			# Cost of electricity
			el_cost     = EL.get_price(t)*np.absolute(u_cs)*10**(-3)*TIMESTEP
			tariff_cost = EL.get_tariff_cost_sim(np.absolute(u_cs))
			total_elcost = total_elcost + el_cost + tariff_cost

			# Cost of power loss
			P_loss = cs_cell_power_loss*u_cs
			P_loss_cost = P_loss_cost + EL.get_price(t)*P_loss*10**(-3)*TIMESTEP + EL.get_tariff_cost_sim(P_loss)

			# Cost of grid load
			if u_cs >= 1000*10**3 and EL.get_net_exchange(t) < 0:
				#total_load_cost = total_load_cost + (-EL.get_net_exchange(t)/1000)*load_factor*EL.get_tariff_cost(np.absolute(u_cs))*TIMESTEP
				total_load_cost = total_load_cost + EL.get_penalty_cost(t, u_cs)*TIMESTEP

		elif mode == 1:
			# Cost of electricity
			el_cost      = EL.get_price(t)*np.absolute(u_grid)*10**(-3)*TIMESTEP
			tariff_cost  = EL.get_tariff_cost_sim(np.absolute(u_grid))
			total_elcost = total_elcost + el_cost + tariff_cost

			# Cost of power loss
			P_loss_cs    = cs_cell_power_loss*u_cs
			P_loss_ferry = ferry_cell_power_loss*(u_cs_ + u_grid)
			P_loss = P_loss_cs + P_loss_ferry
			P_loss_cost = P_loss_cost + EL.get_price(t)*P_loss*10**(-3)*TIMESTEP + EL.get_tariff_cost_sim(P_loss)

			# Cost of grid load
			if u_grid >= 1000*10**3 and EL.get_net_exchange(t) < 0:
				# total_load_cost = total_load_cost + (-EL.get_net_exchange(t)/1000)*load_factor*EL.get_tariff_cost(np.absolute(u_grid))*TIMESTEP
				total_load_cost = total_load_cost + EL.get_penalty_cost(t, u_grid)*TIMESTEP

		else:
			# Cost of electricity
			el_cost      = EL.get_price(t)*np.absolute(u_grid)*10**(-3)*TIMESTEP
			tariff_cost  = EL.get_tariff_cost_sim(np.absolute(u_grid))
			total_elcost = total_elcost + el_cost + tariff_cost

			# Cost of power loss
			P_loss = ferry_cell_power_loss*u_grid
			P_loss_cost = P_loss_cost + EL.get_price(t)*P_loss*10**(-3)*TIMESTEP + EL.get_tariff_cost_sim(P_loss)

			# Cost of grid load
			if u_grid >= 1000*10**3 and EL.get_net_exchange(t) < 0:
				#total_load_cost = total_load_cost + (-EL.get_net_exchange(t)/1000)*load_factor*EL.get_tariff_cost(np.absolute(u_grid))*TIMESTEP
				total_load_cost = total_load_cost + EL.get_penalty_cost(t, u_grid)*TIMESTEP

		# Total energy delivered
		total_energy = total_energy + np.absolute(u_cs + u_grid)*TIMESTEP

		# Update Ah-throughput values
		ferry_cell.update_controllable_Ah_throughput()
		ferry_cell.update_effective_controllable_Ah_throughput()

		cs_cell.update_controllable_Ah_throughput()
		cs_cell.update_effective_controllable_Ah_throughput()


	print('Weightning factors: ', my, '\n')
	print('Final SoC: ', ferry_cell.SOC, cs_cell.SOC)

	total_ferry_Ah_eff      = ferry_cell.Ah_eff*NoC_ferry_Ah
	total_ferry_Ah_ctrl     = ferry_cell.Ah_ctrl*NoC_ferry_Ah
	total_ferry_Ah_eff_ctrl = ferry_cell.Ah_eff_ctrl*NoC_ferry_Ah

	total_cs_Ah_eff      = cs_cell.Ah_eff*NoC_cs_Ah
	total_cs_Ah_ctrl     = cs_cell.Ah_ctrl*NoC_cs_Ah
	total_cs_Ah_eff_ctrl = cs_cell.Ah_eff_ctrl*NoC_cs_Ah

	total_Ah_eff      = total_ferry_Ah_eff + total_cs_Ah_eff
	total_Ah_ctrl     = total_ferry_Ah_ctrl + total_cs_Ah_ctrl
	total_Ah_eff_ctrl = total_ferry_Ah_eff_ctrl + total_cs_Ah_eff_ctrl

	total_energy = total_energy*10**(-3)

	# Add fixed cost of network tariff
	total_elcost = total_elcost + (18000*(1/365)*(1/24)*(1/(60)))*tf

	total_cost = total_elcost + P_loss_cost + (c_batt/Gamma)*cs_cell.Ah_eff_ctrl*NoC_cs + (c_batt/Gamma)*ferry_cell.Ah_eff_ctrl*NoC_ferry + total_load_cost

	print('Total effective Ah: ', total_Ah_eff)
	print('Total controllable Ah: ', total_Ah_ctrl)
	print('Total effective controllable Ah: ', total_Ah_eff_ctrl)
	print('Total controllable energy [kWh]: ', total_energy,'\n')

	print('Price of electricity: ', total_elcost)
	print('Cost of power loss: ', P_loss_cost)
	print('Battery aging cost: ', (c_batt/Gamma)*(cs_cell.Ah_eff_ctrl*NoC_cs + ferry_cell.Ah_eff_ctrl*NoC_ferry))
	print('Grid load cost: ', total_load_cost)
	print('Accumulated cost: ', total_cost, '\n')
	
	print('Ratio, El-cost/Ah: ', total_elcost/total_Ah_ctrl)
	print('Ratio, P_loss/Ah: ', P_loss_cost/total_Ah_ctrl)
	print('Ratio, Aging/Ah: ', ((c_batt/Gamma)*(cs_cell.Ah_eff_ctrl*NoC_cs + ferry_cell.Ah_eff_ctrl*NoC_ferry))/total_Ah_ctrl)
	print('Ratio, P_load/Ah: ', total_load_cost/total_Ah_ctrl)
	print('Ratio, Cost/Ah: ', total_cost/total_Ah_ctrl)
	print('\n')

	SOC_ferry = [i*100 for i in SOC_ferry]
	SOC_cs = [i*100 for i in SOC_cs]

	print(U_opt_cs)
	print(SOC_cs)

	U_opt_grid = [i*10**(-3) for i in U_opt_grid]
	U_opt_ferry = [i*10**(-3) for i in U_opt_ferry]
	U_opt_cs = [i*10**(-3) for i in U_opt_cs]
	U_opt_grid[0] = U_opt_grid[1]
	U_opt_ferry[0] = U_opt_ferry[1]
	U_opt_cs[0] = U_opt_cs[1]
	C_ferry[0] = C_ferry[1]

	hours = np.linspace(8, 8.05, len(U_opt_grid))

	hourss = np.linspace(8, 8.05, 6000)
	U_opt_grid_list  = []
	SOC_ferry_list   = []
	U_opt_ferry_list = []
	C_ferry_list     = []
	SOC_cs_list      = []
	U_opt_cs_list    = []
	for i in range(0, 6000):
		indexx = int(i/1000)
		U_opt_grid_list.append(U_opt_grid[indexx+1])
		SOC_ferry_list.append(SOC_ferry[indexx+1])
		SOC_cs_list.append(SOC_cs[indexx+1])
		U_opt_ferry_list.append(U_opt_ferry[indexx+1])
		C_ferry_list.append(C_ferry[indexx+1])
		U_opt_cs_list.append(U_opt_cs[indexx+1])



	fig, axs = plt.subplots(5,1)
	axs[0].plot(hours, SOC_ferry, linewidth='2')
	axs[0].grid(True)
	axs[0].set_title('Ferry battery normal charging [Standard]', fontsize='16')
	axs[0].set_ylabel('SoC [%]', fontsize='16')
	axs[0].set_xticks(np.linspace(8, 8.05, 7))
	axs[0].set_xticklabels(['07:56:30', '07:57:00', '07:57:30', '07:58:00', '07:58:30', '07:59:00', '07:59:30'])

	axs[1].plot(hourss, U_opt_ferry_list, linewidth='2')
	axs[1].plot([8, 8.00001], [0, U_opt_ferry[0]], color='#1f77b4' ,linewidth='2')
	axs[1].grid(True)
	axs[1].set_ylabel('Power [kW]', fontsize='16')
	axs[1].set_xticks(np.linspace(8, 8.05, 7))
	axs[1].set_xticklabels(['07:56:30', '07:57:00', '07:57:30', '07:58:00', '07:58:30', '07:59:00', '07:59:30'])

	axs[2].plot(hourss, C_ferry_list, linewidth='2')
	axs[2].plot([8, 8.00001], [0, C_ferry[0]], color='#1f77b4' ,linewidth='2')
	axs[2].grid(True)
	axs[2].set_ylabel('Current [A]', fontsize='16')
	axs[2].set_xticks(np.linspace(8, 8.05, 7))
	axs[2].set_xticklabels(['07:56:30', '07:57:00', '07:57:30', '07:58:00', '07:58:30', '07:59:00', '07:59:30'])
	

	color = 'tab:blue'
	axs[3].set_ylabel('CS SoC [%]', fontsize='16')
	line1 = axs[3].plot(hours, SOC_cs, color=color, label='CS SoC', linewidth=2.0)
	# ax1.set_ylim([0, 0.9])
	axs[3].tick_params(axis='y', labelcolor=color)
	axs[3].set_xticks(np.linspace(8, 8.05, 7))
	axs[3].set_xticklabels(['07:56:30', '07:57:00', '07:57:30', '07:58:00', '07:58:30', '07:59:00', '07:59:30'])


	ax2 = axs[3].twinx()  # instantiate a second axes that shares the same x-axis

	color = 'tab:red'
	ax2.set_ylabel('CS power [kW]', color=color, fontsize='16')  # we already handled the x-label with ax1
	line2 = ax2.plot(hourss, U_opt_cs_list, color=color, label='CS power', linewidth=2.0)
	ax2.plot([8, 8.00001], [0, U_opt_cs[0]], color=color ,linewidth='2')
	ax2.tick_params(axis='y', labelcolor=color)
	plt.grid(True)

	# Add legend to line
	lns = line1+line2
	labs = [l.get_label() for l in lns]
	plt.legend(lns, labs, loc=0)	

	axs[4].plot(hourss, U_opt_grid_list, linewidth='2')
	axs[4].plot([8, 8.00001], [0, U_opt_grid_list[0]], color='#1f77b4' ,linewidth='2')
	axs[4].grid(True)
	axs[4].set_ylabel('Grid power [kW]', fontsize='16')
	axs[4].set_xticks(np.linspace(8, 8.05, 7))
	axs[4].set_xticklabels(['07:56:30', '07:57:00', '07:57:30', '07:58:00', '07:58:30', '07:59:00', '07:59:30'])
	axs[4].set_xlabel('Time', fontsize='16')
	fig.tight_layout()  # otherwise the right y-label is slightly clipped
	plt.show()
	return None
