## Plot operational profile

import numpy as np
import matplotlib.pyplot as plt

time_m1      = 45
time_acc     = 90
time_transit = 1090
time_decc    = 185
time_m2      = 90
time_quay    = 300

speed_m1      = np.linspace(0, 6, time_m1)
speed_acc     = np.linspace(6, 12, time_acc)
speed_transit = np.linspace(12, 12, time_transit)
speed_decc    = np.linspace(12, 6, time_decc)
speed_m2      = np.linspace(6, 0, time_m2)
speed_quay    = np.linspace(0, 0, time_quay)

load_m1      = [0.75]*time_m1
load_acc     = [0.8]*time_acc
load_transit = [0.42]*time_transit
load_decc    = [0.07]*time_decc
load_m2      = [0.56]*time_m2
load_quay    = [0.14]*time_quay

## Figure
speed            = np.concatenate((speed_m1, speed_acc, speed_transit, speed_decc, speed_m2, speed_quay))
load             = load_m1+load_acc+load_transit+load_decc+load_m2+load_quay
timeseries_speed = np.linspace(0, 1800/60, len(speed))
timeseries_load  = np.linspace(0, 1800/60, len(load))



color = 'tab:blue'
fig, ax1 = plt.subplots()
ax1.set_xlabel('Time [min]', fontsize='16')
ax1.set_ylabel('Power [p.u]', fontsize='16')
line1 = ax1.plot(timeseries_speed, load, color=color, label='Power', linewidth=2.0)
ax1.set_ylim([0, 0.9])
ax1.tick_params(axis='y', labelcolor=color)
plt.title('Generic operational profile', fontsize='16')

ax2 = ax1.twinx()  # instantiate a second axes that shares the same x-axis

color = 'tab:red'
ax2.set_ylabel('Speed [knots]', color=color, fontsize='16')  # we already handled the x-label with ax1
line2 = ax2.plot(timeseries_load, speed, color=color, label='Speed', linewidth=2.0)
ax2.tick_params(axis='y', labelcolor=color)
plt.grid(True)

# Add legend to line
lns = line1+line2
labs = [l.get_label() for l in lns]
plt.legend(lns, labs, loc=0)

fig.tight_layout()  # otherwise the right y-label is slightly clipped
plt.show()